class CM2mdlParser extends Parser;
options {
	buildAST = true;	// uses CommonAST
}

tokens {
	BOOL_FUNC;
	VECTOR_IMG;
	MACHINE_LIST_ENTRIES_IMG;
	LOCATION_LIST_ENTRIES_IMG;
	TRANSITION_LIST_ENTRIES_IMG;
	HALFSPACE_LIST_ENTRIES_IMG;
	ODE_RHS_IMG;
	EXPR_IMG;
	U_EXPR_IMG;
	U_EXPR_IMG2;
	A_ARGUMENTS_IMG;
	A_ARGUMENT1_IMG;
	A_ARGUMENT2_IMG;
	BOOL_FUNC_LIST_IMG;
	L_ARGUMENT_1_IMG;
	L_ARGUMENT_2_IMG;
	 // imaginary token
}

model
	:	modelname LEFTBRACE^ variables parameters
	        halfspace_list
	        machine_list
	        analysis_region
	        initial_set
	        RIGHTBRACE
	;

modelname
	:	STRING
	;

variables
        :   VARIABLES^ COLON! nnumber SEMI!
        ;
   
parameters
        :   PARAMETERS COLON^ nnumber SEMI! 
        |
        ;

halfspace_list
        :   HALFSPACES^ LEFTBRACE! halfspace halfspace_list_entries 
                                                 RIGHTBRACE!
        ;

halfspace_list_entries
        :   halfspace halfspace_list_entries
        {#halfspace_list_entries= #([HALFSPACE_LIST_ENTRIES_IMG, "HALFSPACE_LIST_ENTRIES_IMG"], #halfspace_list_entries);}
        |
        ;
        
halfspace
        :   STRING COLON^ LEFTBRACKET! vector RIGHTBRACKET! 
                  COMMA! relation COMMA! fnumber SEMI!
        ;

machine_list
        :   MACHINE^ LEFTBRACE! machine_list_entries RIGHTBRACE!
        ;

machine_list_entries
        :  machine machine_list_entries 
         {#machine_list_entries= #([MACHINE_LIST_ENTRIES_IMG, "MACHINE_LIST_ENTRIES_IMG"], #machine_list_entries);}
        |
        ;
        
machine
        :   STRING^ LEFTBRACE! variable_list event_list location_list 
                initial_location RIGHTBRACE!
        ;

event_list : EVENTS^ COLON! input_event_list SEMI output_event_list SEMI!
        |
        ;

input_event_list : STRING^ event_list_entries
        |
        ;

output_event_list : STRING^ event_list_entries
        |
        ;

event_list_entries: COMMA! STRING^ event_list_entries
        |
        ;

variable_list
        :   VARIABLE^ COLON! fnumber fnumber_list_entries
                          SEMI!
        |
                          
        ;

location_list
        :   LOCATIONS^ LEFTBRACE! location_list_entries
                          RIGHTBRACE!
        ;

location_list_entries
        : location location_list_entries
        {#location_list_entries= #([LOCATION_LIST_ENTRIES_IMG, "LOCATION_LIST_ENTRIES_IMG"], #location_list_entries);}
        |
        ;
        
initial_location
        :   INITIAL_LOCATION^ COLON! STRING SEMI! 
        ;

location
        :   STRING^ LEFTBRACE! behaviour invariant
           parameter_constraint transition_list RIGHTBRACE!
        ;

behaviour
        :   INTEGRATOR^ COLON! rate_list
        |   LINEAR^ COLON! linearmap
        | NONLINEAR^ LEFTBRACE! nonlinearmap RIGHTBRACE!
        | 
        ;

rate_list
        :   fnumber fnumber_list_entries SEMI^
        ;
        
linearmap
        :   LEFTBRACKET! vector linearmap_vectors RIGHTBRACKET!
            COMMA^
            LEFTBRACKET! fnumber linearmap_fnumbers RIGHTBRACKET!
            SEMI!
        ;


linearmap_vectors
        : SEMI^ vector linearmap_vectors
        |
        ;
        
linearmap_fnumbers
        :  SEMI^ fnumber linearmap_fnumbers
        |  
        ;
        
vector
        :   fnumber fnumber_list_entries
           {#vector= #([VECTOR_IMG, "VECTOR_IMG"], #vector);}
        ;

fnumber_list_entries
        :  COMMA^  fnumber fnumber_list_entries
        | 
        ;
        
nonlinearmap
        :   ode_rhs SEMI^ nonlinearmap
        |
        ;

ode_rhs
        :  OP^  expr
        |  expr
        {#ode_rhs= #([ODE_RHS_IMG, "ODE_RHS_IMG"], #ode_rhs);}
        ;

expr
        :  u_expr expr_list
        {#expr= #([EXPR_IMG, "EXPR_IMG"], #expr);}
        ;

expr_list
        :  OP^ u_expr expr_list
        |
        ;
u_expr
        :   function
        {#u_expr= #([U_EXPR_IMG, "U_EXPR_IMG"], #u_expr);}
        |   fnumber
        {#u_expr= #([U_EXPR_IMG2, "U_EXPR_IMG2"], #u_expr);}
        |   LEFTPAREN^ expr RIGHTPAREN
        ;

function
        :   STRING^ LEFTPAREN! a_arguments RIGHTPAREN!
        ;

a_arguments
        :   a_argument a_arguments_list
        {#a_arguments= #([A_ARGUMENTS_IMG, "A_ARGUMENTS_IMG"], #a_arguments);}
        ;

a_arguments_list
        :   COMMA^ a_argument a_arguments_list
        |
        ;
        
a_argument
        :   fnumber
        {#a_argument= #([A_ARGUMENT1_IMG, "A_ARGUMENT1_IMG"], #a_argument);}
        |   function
        {#a_argument= #([A_ARGUMENT2_IMG, "A_ARGUMENT2_IMG"], #a_argument);}
        ;

invariant
        :  INVARIANT^ COLON! boolean_function SEMI!
        | 
        ;

boolean_function
        :   l_argument bool_func_list
            {#boolean_function= #([BOOL_FUNC, "BOOL_FUNC"], #boolean_function);}
        ;

bool_func_list
        :  operator l_argument bool_func_list
         {#bool_func_list= #([BOOL_FUNC_LIST_IMG, "BOOL_FUNC_LIST_IMG"], #bool_func_list);}
        |
        ;
        
operator
        :  AND^ 
        |  OR^
        ;

l_argument
        :   LEFTPAREN^ l_argument RIGHTPAREN 
                  l_argument_one
        |   NOT^ l_argument
        |   STRING^ l_argument_two
        ;

l_argument_one
        :  operator l_argument l_argument_one
        {#l_argument_one= #([L_ARGUMENT_1_IMG, "L_ARGUMENT_1_IMG"], #l_argument_one);}
        |
        ;
        
l_argument_two
        : operator l_argument
        {#l_argument_two= #([L_ARGUMENT_2_IMG, "L_ARGUMENT_2_IMG"], #l_argument_two);}
        |
        ;
        
relation
        :  SMALLER^
        |  GREATER^
        |  EQUAL^
        ;

parameter_constraint
        :  PAR_CONSTRAINT^ COLON! boolean_function SEMI!
        |
        ;

transition_list
        :   TRANSITIONS^ LEFTBRACE! transition transition_list_entries
                  RIGHTBRACE!
        |
        ;

transition_list_entries
        : transition transition_list_entries
        {#transition_list_entries= #([TRANSITION_LIST_ENTRIES_IMG, "TRANSITION_LIST_ENTRIES_IMG"], #transition_list_entries);}
        |
        ;
        
transition
        :   STRING^ LEFTBRACE! target guard reset
                clock_flag ievent oevent RIGHTBRACE!
        ;

target
        :  TARGET^ COLON! STRING SEMI!
        ;

guard
        :  GUARD^ COLON! boolean_function SEMI!
        ;

reset
        :   RESET^ COLON! linearmap 
        | 
        ;

clock_flag
        :   CLOCK^ COLON! STRING SEMI!
        |
        ;

ievent : IEVENT^ COLON! STRING SEMI!
        |
        ;
        
oevent : OEVENT^ COLON! STRING SEMI!
        |
        ;
        
analysis_region
        :   ANALYSIS_REGION^ COLON! boolean_function SEMI!
        ;

initial_set
        :   INITIAL_SET^ COLON! boolean_function SEMI!
        ;

fnumber
        :   OP^ FLOAT
        | FLOAT
        ;

nnumber
        :  OP^ FLOAT
        | FLOAT
        ;



class CM2mdlLexer extends Lexer;
options
{
   k = 5;
   testLiterals = true;
}
tokens{
//CheckMate used terminals
MODEL 	= "MODEL"	;
VARIABLES  = "VARIABLES" ;
HALFSPACES = "HALFSPACES" ;
PARAMETERS  = "PARAMETERS" ;
PAR_CONSTRAINT  = "PAR_CONSTRAINT" ;
CLOCKS  = "CLOCKS" ;
LOCATIONS  = "LOCATIONS" ;
TRANSITIONS  = "TRANSITIONS" ;
INTEGRATOR  = "INTEGRATOR" ;
LINEAR  = "LINEAR" ;
NONLINEAR  = "NONLINEAR" ;
INVARIANT  = "INVARIANT" ;
ANALYSIS_REGION  = "ANALYSIS_REGION" ;
INITIAL_SET  = "INITIAL_SET" ;
TARGET = "TARGET" ;
GUARD = "GUARD" ;
CLOCK = "CLOCK" ;
RESET = "RESET" ;
MACHINE = "MACHINE" ;  // new token
INITIAL_LOCATION = "INITIAL_LOCATION" ; //new token
VARIABLE = "VARIABLE"; //
IEVENT = "IEVENT"; // input events
OEVENT = "OEVENT" ; // output events
EVENTS = "EVENTS";
OEVTS = "OEVTS";
IEVTS = "IEVTS";
}

WS	:	(' '
	|	'\t'
	|	'\n'
	|	'\r')
		{ _ttype = Token.SKIP; }
	;
        
LEFTPAREN:	'('
	;

RIGHTPAREN:	')'
	;

LEFTBRACKET:	'['
	;

RIGHTBRACKET:	']'
	;

LEFTBRACE:	'{'
	;

RIGHTBRACE:	'}'
	;

SEMI:	';'
	;

COLON:  ':'
        ;

COMMA:  ','
        ;
        
AND:  '&'
        ;

OR:  '|'
        ;

        
SMALLER:  '<'
        ;

GREATER:  '>'
        ;

NOT     :  '~'
        ;
        
OP:      '+'
        | '-'
        | '*'
        | '/'
        ;

SL_COMMENT : 
	"//" 
	(~'\n')* '\n'
	{ _ttype = Token.SKIP; newline(); }
	;

ML_COMMENT
	:	"/*"
		(	{ LA(2)!='/' }? '*'
		|	'\n' { newline(); }
		|	~('*'|'\n')
		)*
		"*/"
			{ $setType(Token.SKIP); }
	;
     
FLOAT   options{ testLiterals = false;}
        : ( INT ('.' INT ( ('E'|'e')('-'|'+')? INT)? )? )
        ;
        
protected
DIGIT   options{ testLiterals = false;}
	:	'0'..'9'
	;

protected
LETTER  options{ testLiterals = false;}
        :     'a'..'z'
        |     'A'..'Z'
        |     '_'
        ;
        
protected
INT	:	(DIGIT)+
	;
	
STRING  options{ testLiterals = true;}
        :       LETTER(LETTER|DIGIT)*
        ;

