import java.io.*;
import java.lang.*;
import java.util.*;
import antlr.CommonAST;
import antlr.collections.AST;
import antlr.DumpASTVisitor;
import antlr.RecognitionException;
import antlr.TokenStreamException;

class CM2mdl{
	public static void main(String[] args) {
		System.out.println("This is CM2mdl version 1.0");
		System.out.println("");
		try {
			if(args.length!=1)
			{
               System.out.println("Input filename required ");

               System.exit(1);
			}

            int len = args[0].length();
            //System.out.println(args[0]);
            //System.out.println(len);

			if(args[0].charAt(len - 1)!='m' || args[0].charAt(len - 2)!='c'
			       || args[0].charAt(len - 3)!='.'){
			   System.out.println(args[0]+": Not a valid .cm file");
			   System.exit(1);
		    }
            try{
               FileInputStream fin = new FileInputStream(args[0]);

			CM2mdlLexer lexer = new CM2mdlLexer(new DataInputStream(fin));
			lexer.setFilename(args[0]);
			CM2mdlParser parser = new CM2mdlParser(lexer);
			parser.setFilename(args[0]);
			// Parse the input expression
			parser.model();
			CommonAST t = (CommonAST)parser.getAST();
			// Print the resulting tree out in LISP notation

			//System.out.println(t.toStringTree());

			CM2mdlTreeWalker walker = new CM2mdlTreeWalker();
			// Traverse the tree created by the parser
			_model mod = walker.model(t);

//			System.out.println("Before writing setup file **************");
//			print_mod(mod);
			write_setup(mod);


//			System.out.println("Before writing dynamics file **************");
//			print_mod(mod);
			write_dyn(mod);
			// System.out.println("value is "+ mod.toString());


//			System.out.println("Before writing intermediate file **************");
//			print_mod(mod);
			write_intermediate(mod);

			System.out.println("Intermediate file '"+mod.mod+".intr' generated");
			System.out.println("Build the model by \">> build_mdl('"+mod.mod+"')\"");
			}
			catch(FileNotFoundException e)
			{
				System.err.println("File not found");
				System.exit(1);
			}
		}
		catch(TokenStreamException e) {
			System.err.println("exception: "+e);
		}
		catch(RecognitionException e) {
			System.err.println("exception: "+e);
		}
	}


    static void write_intermediate(_model mod)
    {
	   String outf = new String(mod.mod);
	   outf = outf +".intr";

	   try{
		  FileOutputStream fout;

		  fout = new FileOutputStream(outf);

          PrintWriter pw = new PrintWriter(fout, true);

//		  split_states(mod);

		  pw.println("% A CheckMate generated intermediate data file");
          pw.println(mod.mod);

          pw.println("NUM_MACHINES");
          pw.println(mod.mac_list.size());
		  pw.println("VARIABLES");
		  pw.println(mod.var);

		  pw.println("HALFSPACES");
		  int num=0;

		  Enumeration hspaces = mod.hlist.elements();
	      while(hspaces.hasMoreElements()){
			  _half_space hspace;


			  hspace=(_half_space)hspaces.nextElement();

              char[] buf2 = new char[3];
              hspace.name.getChars(0, 3, buf2, 0);

              String bgn = new String(buf2);
              if(!bgn.equals("AR_") && !bgn.equals("ICS")){
				 num++;
			  }
		  }
		  pw.println(num);


		  hspaces = mod.hlist.elements();
	      while(hspaces.hasMoreElements()){
			  _half_space hspace;

			  hspace=(_half_space)hspaces.nextElement();

              char[] buf2 = new char[3];
              hspace.name.getChars(0, 3, buf2, 0);

              String bgn = new String(buf2);
              if(!bgn.equals("AR_") && !bgn.equals("ICS")){
				 pw.println(hspace.name);
			  }
		  }

		  pw.println("MACHINES");
		  pw.println(mod.mac_list.size());

          Enumeration machines = mod.mac_list.elements();

          while(machines.hasMoreElements()){
		     _machine mac;

		     mac=(_machine)machines.nextElement();

		     pw.println("machine");
		     pw.println(mac.name);
		     pw.println(mac.llist.size());
		     pw.println(mac.ini_loc);

		     // New Code for events
		     Enumeration elists = mac.elist.elements();

		     //System.out.println("In new_Ascii.java");
		     //System.out.println(mac.elist.size()+"");

		     if(mac.elist.size()==2){
		         Vector ielist=(Vector)elists.nextElement();
		         Vector oelist=(Vector)elists.nextElement();

		         //System.out.println("Input event vector :"+ielist.size());
		         //System.out.println("Output event vector :"+oelist.size());

		         pw.println("I_EVENTS");
		         pw.println(""+ielist.size());
		         Enumeration ieves = ielist.elements();
		         while(ieves.hasMoreElements()){
				    String ievent = (String)ieves.nextElement();
				    pw.println(ievent);
				 }
		         pw.println("ENDI_EVENTS");

		         pw.println("O_EVENTS");
		         pw.println(""+oelist.size());
		         Enumeration oeves = oelist.elements();
		         while(oeves.hasMoreElements()){
				    String oevent = (String)oeves.nextElement();
				    pw.println(oevent);
				 }
		         pw.println("ENDO_EVENTS");
		     } // End New code for events

		     Enumeration locations = mac.llist.elements();
		     while(locations.hasMoreElements()){
				 _location loc;

				 loc=(_location)locations.nextElement();
				 pw.println("state");
				 pw.println(loc.name);

				 Enumeration transitions=loc.tlist.elements();
				 while(transitions.hasMoreElements()){
                    _transition trans;

                    trans = (_transition)transitions.nextElement();
                    String out_string="";
                    if(trans.iev_flag==1) out_string="1+";
                    else out_string="";

                    String target_state = trans.target;
                    Enumeration searchfortarget = mac.llist.elements();
                    _location tgt = new _location(); // to satisfy the compiler
                    while(searchfortarget.hasMoreElements()){
						tgt = (_location)searchfortarget.nextElement();
						if(tgt.name.trim().equals(target_state.trim())) break;
					}
                    if(tgt.reset_flag==0)
                    {
						/* no reset */
						if(trans.oevent.equals(""))
                           pw.println(out_string+trans.target+";"+trans.guard);
                        else
                           pw.println(out_string+trans.oevent+"*"+trans.target+";"+trans.guard);
					}
					else
					{
	//					System.out.println("trans.reset.vecs.size is not zero!!!");
						if(trans.oevent.equals(""))
						   pw.println(out_string+"reset;"+trans.target+";"+trans.guard);
						else
                           pw.println(out_string+trans.oevent+"*reset;"+trans.target+";"+trans.guard);
					}
				}

			 }

		  }

	   }catch(FileNotFoundException e){
	      System.out.println("exception" + e);
	      System.exit(1);
	   }
	}

// GENERATE THE DYNAMICS m-FILE
    static void write_dyn(_model mod)
    {
	   String outf = new String(mod.mod);
	   outf = outf +"_dyn";

	   try{
	       FileOutputStream fout;

    	   fout = new FileOutputStream(outf+".m");

           PrintWriter pw = new PrintWriter(fout, true);
           String overall_dyn = "";

    	   pw.println(" % A CheckMate generated dynamics M-file");
    	   pw.println("function [sys, type, reset] = "+outf+"(x,u)");

    	   pw.println("");
    	   overall_dyn = get_type(mod);

    //	if(!overall_dyn.equals(" ")){
    	   pw.println("   type = '"+ overall_dyn.toLowerCase()+"';");
    	   pw.println("");

		   pw.println("   reset.A = eye("+(int)mod.var+");");
		   pw.println("   reset.B = zeros("+ (int)mod.var+",1);");
		   pw.println("");

//System.out.println("****************************************");
//System.out.println("mod before calling split states");
//print_mod(mod);
//System.out.println("****************************************");
    	   split_states(mod); // Check for multiple incoming resets and split states
//System.out.println("****************************************");
//System.out.println("mod after returning from splitstates");
//print_mod(mod);
//System.out.println("****************************************");
           Enumeration machines = mod.mac_list.elements();

           int idx = mod.mac_list.size();
	       _machine mac;
		   _location loc=new _location();

           while(machines.hasMoreElements())
           {
		 //     pw.println("   switch u(" + idx + ")");
		//	  System.out.println("size = " + idx + "mod.mac_list.size = " + mod.mac_list.size());
			  mac = (_machine)machines.nextElement();
		      mac = (_machine)mod.mac_list.elementAt(idx-1);

		      int end_flag=0;

              // Write dynamics of every location
		      Enumeration locations = mac.llist.elements();
			  int machine_reset_flag = 0;
			  while(locations.hasMoreElements())
		      {
				  loc = (_location)locations.nextElement();
				  if(loc.reset_flag!=0) machine_reset_flag=1;
			  }

/* The code for generating the RESETs in the dynamics file is modified
   The assumption is that only one machine would be allowed to reset and hence
   the numbers such as "reset_1" etc are not required anymore */

		      locations = mac.llist.elements();
		      int idx2 = mac.llist.size();
		      int  size = idx2;
		      while(locations.hasMoreElements())
		      {
				  loc = (_location)locations.nextElement();

			  if(!loc.bhvr.type.equals(" ")){
				  end_flag=1;

				  if(idx2 == mac.llist.size())
				     pw.println("   switch u(" + idx + ")");

				  pw.println("      case "+idx2);
				  if(overall_dyn.equals("CLOCK"))
				  {
					 pw.println("         sys_"+idx+" = ["+get_string(loc.bhvr.vec)+"]';");
					 //new code for reset
					 if(machine_reset_flag==1){
						if(loc.reset_flag==0)
						{
							pw.println("         reset.A = eye("+(int)mod.var+");");
							pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
						}
						else
						{
							pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
							pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
						}
					 }

				  }
				  else if(overall_dyn.equals("LINEAR"))
				  {
					 if(loc.bhvr.type.equals("INTEGRATOR")){
						// Change to remove the square matrix requirement
						// for the linear case
					  //pw.println("         sys_"+idx+".A = ["+get_stringAzeroes(loc.bhvr.vec.size())+"];");
					    pw.println("         sys_"+idx+".A = ["+get_stringAzeroes_rect((int)mac.vlist.size(),(int)mod.var)+"];");
                        pw.println("         sys_"+idx+".b = ["+ get_string(loc.bhvr.vec) +"]';");

                        // new code for RESET
						if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}
					 	}

					 }
					 else{
						// location behaviour type is linear
						//System.out.println("*****"+get_stringA(loc.bhvr.lmap.vecs));
					    pw.println("         sys_"+idx+".A = ["+ get_stringA(loc.bhvr.lmap.vecs) +"];");
					    pw.println("         sys_"+idx+".b = ["+ get_string(loc.bhvr.lmap.fnumbers) +"]';");

                        // new code for RESET
                        if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}
					 	}

				     }
				  }
				  else if(overall_dyn.equals("NONLINEAR"))
				  {
					 int size2;
					 int idx3;
					 if(loc.bhvr.type.equals("INTEGRATOR")
					    || loc.bhvr.type.equals("NONLINEAR"))
					    idx3 = loc.bhvr.vec.size();
					 else idx3=0;
					 size2 = idx3;

					 if(loc.bhvr.type.equals("INTEGRATOR")){
					     Enumeration nlmap = loc.bhvr.vec.elements();
						 while(nlmap.hasMoreElements())
						 {
						 	Float f = (Float)nlmap.nextElement();
						 	pw.println("         sys_"+idx+".x("+idx3+") = " + f.toString()+" ;");


						 	idx3--;
				     	 }
						if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}
						}
					 }
					 if(loc.bhvr.type.equals("LINEAR")){
					 	idx3= loc.bhvr.lmap.vecs.size();
					 	// must also be equal to the fnumber vector size

					 	Enumeration fnum=loc.bhvr.lmap.fnumbers.elements();
					 	Enumeration rows=loc.bhvr.lmap.vecs.elements();

					 	while(idx3>0){
					 		Vector row = (Vector)rows.nextElement();
					 		Float f = (Float)fnum.nextElement();
					 		pw.println("         sys_"+idx+".x("+idx3+") = [" + get_string(row) + "]*x +"+ f.toString() +" ;");


					 		idx3--;
					 	}

						if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}
						}
					 }
					 if(loc.bhvr.type.equals("NONLINEAR")){
				    	 Enumeration nlmap = loc.bhvr.vec.elements();
				    	 while(nlmap.hasMoreElements())
					     {
				    		String ode_rhs = (String)nlmap.nextElement();
				    		pw.println("         sys_"+idx+".x("+idx3+") = " + ode_rhs+" ;");


				    		idx3--;
				     	 }

						if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}
						}
				     }
				  }
			  }// End the check 'if'

				  idx2--;
			  }

	//		  pw.println("      otherwise");
			  loc = (_location)mac.llist.elementAt(size - 1);
			  if(!loc.bhvr.type.equals(" ")){

			  pw.println("      otherwise");

			  if(overall_dyn.equals("CLOCK"))
			  {
					 pw.println("         sys_"+idx+" = ["+get_string(loc.bhvr.vec)+"]';");

					 if(machine_reset_flag==1){
						if(loc.reset_flag==0)
						{
							pw.println("         reset.A = eye("+(int)mod.var+");");
							pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
						}
						else
						{
							if(loc.reset.fnumbers.size() >0){
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}
						}
					 }
			  }
			  else if(overall_dyn.equals("LINEAR"))
			  {
				 if(loc.bhvr.type.equals("INTEGRATOR")){
					 // Change to remove the square matrix requirement
					 // for the linear case
					 //pw.println("         sys_"+idx+".A = ["+get_stringAzeroes(loc.bhvr.vec.size())+"];");
					 pw.println("         sys_"+idx+".A = ["+get_stringAzeroes_rect((int)mac.vlist.size(),(int)mod.var)+"];");
                     pw.println("         sys_"+idx+".b = ["+ get_string(loc.bhvr.vec) +"]';");

                     // new code for RESET
					 if(machine_reset_flag==1){
						if(loc.reset_flag==0)
						{
							pw.println("         reset.A = eye("+(int)mod.var+");");
							pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
						}
						else
						{
							if(loc.reset.fnumbers.size() >0){
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}						}
					 }
				 }
				 if(loc.bhvr.type.equals("LINEAR")){
					 pw.println("         sys_"+idx+".A = ["+ get_stringA(loc.bhvr.lmap.vecs) +"];");
					 pw.println("         sys_"+idx+".b = ["+ get_string(loc.bhvr.lmap.fnumbers) +"]';");

                     // new code for RESET
					 if(machine_reset_flag==1){
						if(loc.reset_flag==0)
						{
							pw.println("         reset.A = eye("+(int)mod.var+");");
							pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
						}
						else
						{
							if(loc.reset.fnumbers.size() >0){
								pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
								pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
							}						}
					 }
				 }
			  }
			  else if(overall_dyn.equals("NONLINEAR"))
			  {
				 int size2, idx3;
				 if(loc.bhvr.type.equals("INTEGRATOR")
					|| loc.bhvr.type.equals("NONLINEAR"))
				    idx3 = loc.bhvr.vec.size();
				 else idx3=0;
				 size2 = idx3;

					 if(loc.bhvr.type.equals("INTEGRATOR")){
					     Enumeration nlmap = loc.bhvr.vec.elements();
						 while(nlmap.hasMoreElements())
						 {
						 	Float f = (Float)nlmap.nextElement();
						 	pw.println("         sys_"+idx+".x("+idx3+") = " + f.toString()+" ;");

						 	idx3--;
				     	 }

						 if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								if(loc.reset.fnumbers.size() >0){
									pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
									pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
								}
							}
						 }
					 }
					 if(loc.bhvr.type.equals("LINEAR")){
						idx3= loc.bhvr.lmap.vecs.size();
						// must also be equal to the fnumber vector size

						Enumeration fnum=loc.bhvr.lmap.fnumbers.elements();
						Enumeration rows=loc.bhvr.lmap.vecs.elements();

						while(idx3>0){
							Vector row = (Vector)rows.nextElement();
							Float f = (Float)fnum.nextElement();
				    		pw.println("         sys_"+idx+".x("+idx3+") = [" + get_string(row) + "]*x +"+ f.toString() +" ;");

							idx3--;
						}

						 if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								if(loc.reset.fnumbers.size() >0){
									pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
									pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
								}
							}
						 }
					 }
					 if(loc.bhvr.type.equals("NONLINEAR")){
				    	 Enumeration nlmap = loc.bhvr.vec.elements();
				    	 while(nlmap.hasMoreElements())
					     {
				    		String ode_rhs = (String)nlmap.nextElement();
				    		pw.println("         sys_"+idx+".x("+idx3+") = " + ode_rhs+" ;");

				    		idx3--;
				     	 }

						 if(machine_reset_flag==1){
							if(loc.reset_flag==0)
							{
								pw.println("         reset.A = eye("+(int)mod.var+");");
								pw.println("         reset.B = zeros("+ (int)mod.var+",1);");
							}
							else
							{
								if(loc.reset.fnumbers.size() >0){
									pw.println("         reset.A = ["+get_resetA(loc.reset.vecs,(int)mod.var, mac.vlist)+"];");
									pw.println("         reset.B = ["+ get_resetB(loc.reset.fnumbers,(int)mod.var, mac.vlist) +"];");
								}
							}
						 }
				     }
			  }

		      } // end check 'if'
			  if(end_flag==1) pw.println("   end");
			  idx--;
		   }
//	   } // end check for empty dynamics

		   // Finish the file
		   int size = mod.mac_list.size();
		   idx=1; //New line
		   machines = mod.mac_list.elements();
		   int lin_flag=0;
		   String pA = "", pB ="", rA="", rB="";

           //System.out.println("machine list "+size);
		   while(machines.hasMoreElements()){
               mac = (_machine)machines.nextElement();



			   Float[] indices = new Float[mac.vlist.size()];
			   int len_indices = mac.vlist.size();

			 //  for(int i=len_indices-1;i>=0;i--){
			   for(int i=(int)mod.var;i>=1;i--){
				   //search for this variable in the var list of machine
				   Enumeration varlist = mac.vlist.elements();

				   int num_var=mac.vlist.size();

				   int idx3=1; int flag=0;
				   while(varlist.hasMoreElements())
				   	{
				   		Float f=(Float)varlist.nextElement();

				   		if(((int)f.floatValue())==i) {flag=1; break;}
				   		idx3++;
		   			}
		   			if(flag==1){
				   //indices[i]=(Float)varlist.nextElement();

				   		if(overall_dyn.equals("CLOCK")){
							  //pw.println("   sys("+indices[i]+",1) = sys_"+idx+"("+indices[i]+",1);");
							  //  System.out.println(i+" "+len_indices);
							    int temp=num_var - idx3 + 1;
				   				pw.println("   sys("+i+",1) = sys_"+idx+"("+temp+",1);");
				 		  }
					   	else if(overall_dyn.equals("LINEAR")){
						   int temp=num_var - idx3 + 1;
						   pw.println("   sys.A("+i+",:) = sys_"+idx+".A("+temp+",:);");
						   pw.println("   sys.b("+i+",:) = sys_"+idx+".b("+temp+",:);");
				 		}
				 		else if(overall_dyn.equals("NONLINEAR")){
				   		   int size2, sysmaplen;
				   		   if(loc.bhvr.type.equals("INTEGRATOR")
				    		|| loc.bhvr.type.equals("NONLINEAR"))
				    		    sysmaplen = loc.bhvr.vec.size();
				    		else sysmaplen=loc.bhvr.lmap.fnumbers.size();

				      		for(int j=sysmaplen-1;j>=0;j--){
								int temp = num_var - idx3 + 1;
					     		pw.println("   sys("+i+",1) = sys_"+idx+".x("+temp+");");
					  		}
				   		}
					}
			   }


			   if(overall_dyn.equals("LINEAR")){
				   Enumeration locations = mac.llist.elements();
		           _location temp_loc = (_location)locations.nextElement();

		           //int index = mod.mac_list.size() - idx + 1;

			       if(lin_flag==0){
					   lin_flag = 1;
				       pA = "sys.A = ";
				       pB = "sys.b = ";
				       rA = "reset.A = ";
				       rB = "reset.B = ";

		               if(!temp_loc.bhvr.type.equals(" ")){
				        //   pA = pA + " sys_"+ index +".A ";
				        //   pB = pB + " sys_"+ index +".b ";
				           rA = rA + " reset_"+ idx +".A ";
				           rB = rB + " reset_"+ idx +".B ";
					   }
					   else lin_flag = 0;
				   }
				   else{
		               if(!temp_loc.bhvr.type.equals(" ")){
				         //  pA = pA + "+ sys_"+ index +".A ";
				         //  pB = pB + "+ sys_"+ index +".b ";
				           rA = rA + "+ reset_"+ idx +".A ";
				           rB = rB + "+ reset_"+ idx +".B ";
					   }
				   }
			   }


		       idx++;
		   }

		   if(!pA.equals("")){
			 //  pw.println("   "+pA+";");
			 //  pw.println("   "+pB+";");
			 //  pw.println("   "+rA+";");
			 //  pw.println("   "+rB+";");
		   }

		   pw.println("return");


	   }
	   catch(FileNotFoundException e){
	      System.out.println("Exception in function write_dyn: " + e);
	      System.exit(1);
	   }
    }

    static String get_stringAzeroes(int size){
	   int i, j;

       String ret_val="";
	   for(i=0;i<size;i++){

		   String temp="";
		   for(j=0;j<size;j++){
			  if(temp.equals("")) temp = "0";
		      else temp = "0, " + temp;
		   }

		   if(ret_val.equals("")) ret_val=temp;
		   else ret_val = temp + ";" + ret_val;
	   }
	   return ret_val;
	}

    static String get_stringAzeroes_rect(int rows,int size){
	   int i, j;

       String ret_val="";
	   for(i=0;i<rows;i++){

		   String temp="";
		   for(j=0;j<size;j++){
			  if(temp.equals("")) temp = "0";
		      else temp = "0, " + temp;
		   }

		   if(ret_val.equals("")) ret_val=temp;
		   else ret_val = temp + ";" + ret_val;
	   }
	   return ret_val;
	}

    static String get_resetA(Vector vec, int num, Vector vlist){
	   // returns the string representation of a matrix
	   // i.e a vector of vectors

	   String ret_val = "";
	   Enumeration vc = vec.elements();

		for(int i=1;i<=num;i++)
		{
			// search to see whether the variable i is in vlist
		   int idx=1;
			int flag=0;
		   Enumeration vars = vlist.elements();
		   while(vars.hasMoreElements())
		   {
			   Float f=(Float)vars.nextElement();

			   if(((int)f.floatValue())==i) {flag=1; break;}
			   idx++;
		   }
		   if(flag==0){
			   // this variable not present so it remains unchanged
			  if(ret_val.equals("")) ret_val = get_zeros(i-1) + " 1 " +get_zeros(num-i);
			  else ret_val = ret_val + ";" + get_zeros(i-1) + " 1 " +get_zeros(num-i) ;

			 // System.out.print(i + " ");
			 // System.out.println(get_zeros(i-1) + " 1 " +get_zeros(num-i-1) );
		   }
		   else{
			  int size = vec.size();
			//	System.out.print(i+" ");
			  Vector v = (Vector)vec.elementAt(size-1);
			  vec.removeElementAt(size-1);
			//	System.out.println(get_string(v));
			  if(ret_val.equals("")) ret_val = get_string(v);
			  else ret_val = ret_val + ";" + get_string(v) ;
		   }
		}

	   return ret_val;
	}

	// This function is used by the generator of the dynamics file
    static String get_resetB(Vector vec, int num, Vector vlist){

	   String ret_val = "";
	   Enumeration vc = vec.elements();

/*		   Enumeration vars = vlist.elements();
		   while(vars.hasMoreElements())
		   {
			   System.out.print(vars.nextElement() + " ");
			   System.out.println("**********************");
		   }*/
		// if(vec.size()==0) System.out.println("Empty vector passed to method");

		for(int i=1;i<=num;i++)
		{
			// search to see whether the variable i is in vlist
		   int idx=1;
			int flag=0;
		   Enumeration vars = vlist.elements();
		  // System.out.print(i+ " ");
		   while(vars.hasMoreElements())
		   {
			   Float f=(Float)vars.nextElement();

			   if(((int)f.floatValue())==i) {flag=1; break;}
			   idx++;
		   }
		   if(flag==0){
			   // this variable not present so it remains unchanged
			  if(ret_val.equals("")) ret_val = " 0 ";
			  else ret_val = ret_val +"; 0 ";
			//  System.out.println("not found ");
		   }
		   else{
			  int size = vec.size();
			//   System.out.println("size " + size);
			  Float f = (Float)vec.elementAt(size-1);
			  vec.removeElementAt(size-1);
			//  System.out.println("removed " + f);

			  if(ret_val.equals("")) ret_val = f.toString();
			  else ret_val = ret_val + ";" + f.toString()  ;
			//  System.out.println("found ");
		   }
		}

	   return ret_val;
    }

	// This function is used by the generator of the dynamics file
    static String get_zeros(int num){
       String ret_val = "";

       for(int i=0;i<num;i++)
       {
		  ret_val = "0 " + ret_val;
	   }

	   return ret_val;
    }

    static String get_stringA(Vector vec){
	   // returns the string representation of a matrix
	   // i.e a vector of vectors

	   String ret_val = "";

	   Enumeration vc = vec.elements();
	   while(vc.hasMoreElements())
	   {
		  Vector v = (Vector)vc.nextElement();

		  if(ret_val.equals("")) ret_val = get_string(v);
          else ret_val = get_string(v) + ";"+ ret_val;
	   }

	   return ret_val;
	}

	// This function is used by the generator of the dynamics file to
	// get the string representation of a vector of floats
    static String get_string(Vector vec){
       String ret_val = "";

       Enumeration flots = vec.elements();
       while(flots.hasMoreElements())
       {
		  Float f = (Float)flots.nextElement();

		  ret_val = f.toString() + " " + ret_val;
	   }

	   return ret_val;
    }

    // This function is used by the generator of the dynamics file to
	// get the string representation of a vector of floats
	static String get_string_intg(_location loc, Vector vlist, int n){
	   String ret_val = "";

	   Vector vec = loc.bhvr.vec;
//System.out.println(n+" " +vlist.size()+ " " + loc.bhvr.vec.size());
	   for(int i=n;i>=1;i--)
	   {
		   // search for this variable in the machine's list
		   // if found plug in value else plug in zero

		   int flag=0;
		   int idx=1;

		   Enumeration vars = vlist.elements();
		   while(vars.hasMoreElements())
		   {
			   Float f=(Float)vars.nextElement();

			   if(((int)f.floatValue())==i) {flag=1; break;}
			   idx++;
		   }

		   if(flag==0)
		   {
			   //System.out.println("In get_sting_intg .. found flag=0");
			   ret_val = "0 " + ret_val;
		   }
		   else{
			   // pick the value from 'vec'
			   //System.out.println("In get_sting_intg .. found flag=1 , idx="+idx);
			   int idx2 = 1;
			   Enumeration flots = vec.elements();
			   while(flots.hasMoreElements())
			   {
			   		Float f = (Float)flots.nextElement();

			   		if(idx2 == idx) ret_val = f.toString() + " " + ret_val;
			   		idx2++;
				}
		   }
		}

		return ret_val;
    }

    // This function is used by the generator of the dynamics file to
	// get the string representation of a vector of floats
	static String get_stringZ(Vector vec){
	    String ret_val = "";

	    Enumeration flots = vec.elements();
	    while(flots.hasMoreElements())
	    {
			Float f = (Float)flots.nextElement();

			ret_val = "0 " + ret_val;
		}

		return ret_val;
    }

// This function is used by the generator of the dynamics file to
// get the type
    static String get_type(_model mod)
    {
		// Check the overall dynamics of the parse tree

		int all_linear = 1, all_INTEGRATOR = 1, all_empty=1;
		String ret_val = "";
		_machine mac;

		Enumeration machines = mod.mac_list.elements();
		// Get the machines in the model's machine_list
		while(machines.hasMoreElements())
		{
           mac=(_machine)machines.nextElement();

           _location loc;
           Enumeration locations = mac.llist.elements();
           while(locations.hasMoreElements())
           {
			   loc = (_location)locations.nextElement();

			   _behaviour bhvr = loc.bhvr;

			   if(bhvr.type.equals("NONLINEAR"))
			      all_linear = 0;
			   if(!bhvr.type.equals("INTEGRATOR"))
			      all_INTEGRATOR = 0;

			   if(!bhvr.type.equals(" "))
			      all_empty=0;
		   }
		}


		if(all_INTEGRATOR == 1) ret_val = "CLOCK";
		else if(all_linear == 1) ret_val = "LINEAR";
		else ret_val = "NONLINEAR";

		return ret_val;
	}

// GENERATE THE SETUP m-FILE
	static void write_setup(_model mod)
	{
       String outf = new String(mod.mod);

       try{
    	   FileOutputStream fout = new FileOutputStream(outf+"_setup.m");

           PrintWriter pw = new PrintWriter(fout, true);

    	   pw.println(" % A CheckMate generated setup M-file");
    	   pw.println("evalin('base','global_var');");
    	   pw.println(" % Create global variables");
    	   pw.println("");

          String ini_set="", ini_set_2="", ana_reg="", ana_reg_2="";

	      Enumeration hspaces = mod.hlist.elements();
	      while(hspaces.hasMoreElements())
	      {
		      _half_space hspace = (_half_space)hspaces.nextElement();

              int rel_flag;
			  Float f;
			  float dummy;

              char[] buf2 = new char[3];
              hspace.name.getChars(0, 3, buf2, 0);

              String bgn = new String(buf2);
              if(bgn.equals("AR_"))
              {
				 String ini = "";

                 if(hspace.relation=='<') rel_flag = 1;
                 else rel_flag = -1;

                 Enumeration vec = hspace.vec.elements();
				 while(vec.hasMoreElements())
				 {
				 	f = (Float)vec.nextElement();
				 	dummy = rel_flag * f.floatValue();
				 	if(ini.equals("")) ini = dummy + "";
				 	else ini = dummy + ", " + ini ;
			     }

			     if(ana_reg.equals("")) ana_reg = ini;
			     else ana_reg = ini  + " ; " + ana_reg;

			     if(ana_reg_2.equals("")) ana_reg_2 = rel_flag*hspace.fnumber + "";
			     else ana_reg_2 = rel_flag*hspace.fnumber + ";"  + ana_reg_2 ;
		      }
              else if(bgn.equals("ICS"))
              {
				 String ini="";

                 if(hspace.relation=='<') rel_flag = 1;
                 else rel_flag = -1;

                 Enumeration vec = hspace.vec.elements();
				 while(vec.hasMoreElements())
				 {
				 	f = (Float)vec.nextElement();
				 	dummy = rel_flag * f.floatValue();
				 	if(ini.equals("")) ini = dummy + "";
				 	else ini = dummy + ", " + ini ;
			     }

			     if(ini_set.equals("")) ini_set = ini;
			     else ini_set = ini  + " ; " + ini_set;

			     if(ini_set_2.equals("")) ini_set_2 = rel_flag*hspace.fnumber + "";
			     else ini_set_2 = rel_flag*hspace.fnumber + ";"  + ini_set_2 ;
		      }
		      else{
                 pw.print(hspace.name+" = linearcon([],[],[");

                 String fvec="";

                 if(hspace.relation=='<') rel_flag = 1;
                 else rel_flag = -1;

                 Enumeration vec = hspace.vec.elements();
                 while(vec.hasMoreElements())
                 {
					 f = (Float)vec.nextElement();
					 dummy = rel_flag * f.floatValue();
					 if(fvec.equals("")) fvec = dummy + "";
					 else fvec = dummy + ", " + fvec ;
			     }
			     pw.print(fvec);

			     pw.print("],");
			     dummy = hspace.fnumber;
			     dummy = rel_flag * dummy;
			     pw.print(dummy);
			     pw.println(");");

			     pw.println("assignin('base','"+hspace.name+"',"+hspace.name+");");
		      }
          }

          pw.println("initial_set = {linearcon([], [], [" + ini_set + "], [" + ini_set_2 + " ])};");
          pw.println("assignin('base','initial_set',initial_set);");

          pw.println("analysis_region = linearcon([], [], [" + ana_reg + "], [" + ana_reg_2 + " ]);");
          pw.println("assignin('base','analysis_region',analysis_region);");

          pw.println("");
          pw.println("GLOBAL_SYSTEM = '" + outf + "';");

       }
	   catch(Exception e){
	      System.out.println("Exception" + e);
       }
    }

/* Trverse the transition list for each location
 * creating new locations for those locations which have more than
 * one incoming reset OR
 * one ncoming reset but more than 1 total incoming transitions
 */
    static void split_states(_model mod){
//		print_mod(mod);
		enum_resets(mod);

		Enumeration macs = mod.mac_list.elements();

		while(macs.hasMoreElements())
	    {
			_machine mac=(_machine) macs.nextElement();

			Enumeration locs = mac.llist.elements(); /*location list for this machine */
			Vector new_llist= new Vector(3,2);

			while(locs.hasMoreElements())
			{
				_location loc=(_location)locs.nextElement();

				/* traverse the transition list for this location
				 * simultaneously splitting target locations as needed
				 */

				Enumeration trans=loc.tlist.elements();
				while(trans.hasMoreElements())
				{
					_transition tran=(_transition)trans.nextElement();

					if(tran.reset.vecs.size()>0)
					{

						/* Reset non-empty */
						String tgt = tran.target;
						tgt = tgt.trim();

						/* find the target location and adjust counters*/
						Enumeration locs_2 = mac.llist.elements(); /*location list for this machine */


						while(locs_2.hasMoreElements())
						{
							_location loc_2 = (_location)locs_2.nextElement();
							if(tgt.equals(loc_2.name))
							{
								/* check whether this is the only incoming transition for
								 * this state
								 */
								 if(loc_2.entering_trans==1)
								 {
									 /* No new location to be created
									  * move the reset into loc_2 */

									 loc_2.reset=tran.reset;
									 loc_2.reset_flag=1;
								 }
								 else
								 {
									 // Don't process self transitions on the first pass.
									 if(!tgt.equals(loc.name.trim())){
									    /*create a new location for this reset transition*/

									    _location new_loc=new _location(loc_2);

									    /*the new location has only one incoming transition */
									    new_loc.entering_resets=1;
									    new_loc.entering_trans=1;

									    /*decrement the unumber of entering resets for locs2 */
									    loc_2.entering_resets = loc_2.entering_resets - 1;
									    // not valid for self loop when the state is evaluated first

									    /*change the new location's name*/
									    new_loc.name=loc.name+"_to_"+new_loc.name;

									    /*move the reset to new location*/
									    new_loc.reset=tran.reset;
									    new_loc.reset_flag=1;

									    /*redirect this transition to new location */
									    tran.target=new_loc.name;

									    new_llist.addElement(new_loc);
									}
								 }
							}
						}
					}
				}
			}


		   locs = mac.llist.elements(); /*location list for this machine */

		   while(locs.hasMoreElements())
		   {
			   _location loc=(_location)locs.nextElement();

			   /* traverse the transition list for this location
			    * simultaneously splitting target locations as needed
			    */

			   Enumeration trans=loc.tlist.elements();
			   while(trans.hasMoreElements())
			   {
				   _transition tran=(_transition)trans.nextElement();

				   if(tran.reset.vecs.size()>0)
				   {

					   /* Reset non-empty */
					   String tgt = tran.target;
					   tgt = tgt.trim();

					   /* find the target location and adjust counters*/
					   Enumeration locs_2 = mac.llist.elements(); /*location list for this machine */


					   while(locs_2.hasMoreElements())
					   {
						   _location loc_2 = (_location)locs_2.nextElement();
						   if(tgt.equals(loc_2.name))
						   {
							   /* check whether this is the only incoming transition for
							   * this state
							   */
							   if(loc_2.entering_trans==1)
							   {
							   	   /* No new location to be created
								    * move the reset into loc_2 */

								   loc_2.reset=tran.reset;
								   loc_2.reset_flag=1;
							   }
							   else
							   {
								   // process self transitions
								   if(tgt.equals(loc.name.trim()))
								   {
									   /*create a new location for this reset transition*/

									   _location new_loc=new _location(loc_2);

									   /*the new location has only one incoming transition */
									   new_loc.entering_resets=1;
									   new_loc.entering_trans=1;

									   /*decrement the unumber of entering resets for locs2 */
									   loc_2.entering_resets = loc_2.entering_resets - 1;
									   // not valid for self loop when the state is evaluated first

									   /*change the new location's name*/
									   new_loc.name=loc.name+"_to_"+new_loc.name;

									   /*move the reset to new location*/
									   new_loc.reset=tran.reset;
									   new_loc.reset_flag=1;

									   /*redirect this transition to new location */
									   tran.target=new_loc.name;

									   new_llist.addElement(new_loc);
								   }
							   }
						   }
					   }
				   }
			   }
		   }
		   /* Add the new locations to the location list of the machine */
		   Enumeration new_locs=new_llist.elements();
		   while(new_locs.hasMoreElements())
		   {
			  _location new_loc=(_location)new_locs.nextElement();

			  /*insert at the beginning (for some implementation reason) */
			  mac.llist.insertElementAt(new_loc,0);
		   }

		}

//		print_mod(mod);
    }

/* Traverse the transition list of each location for all machines
 * and count the number of incoming total transitions
 * and the number of incoming reset transitions for each location
 */
    static void enum_resets(_model mod){
	    Enumeration macs = mod.mac_list.elements();

	    while(macs.hasMoreElements())
	    {
			_machine mac=(_machine) macs.nextElement();

			Enumeration locs = mac.llist.elements(); /*location list for this machine */
			while(locs.hasMoreElements())
			{
				_location loc=(_location)locs.nextElement();

				/* traverse the transition list for this location
				 * simultaneously evaluate the reset counter for the
				 * target locations */

				Enumeration trans=loc.tlist.elements();
				while(trans.hasMoreElements())
				{
					_transition tran=(_transition)trans.nextElement();

					if(tran.reset.vecs.size()>0)
					{

						/* Reset non-empty */
						String tgt = tran.target;
						tgt = tgt.trim();

						/* find the target location and adjust counters*/
						Enumeration locs_2 = mac.llist.elements(); /*location list for this machine */
			            while(locs_2.hasMoreElements())
			            {

							_location loc_2 = (_location)locs_2.nextElement();
							if(tgt.equals(loc_2.name))
							{
								//System.out.println("Found a RESET.... from loc "+loc.name+" to loc: "+tran.target);
								loc_2.entering_trans++;
								loc_2.entering_resets++;
							}
						}
					}
					else{
						/* No reset associted with this transition */

						String tgt = tran.target;
						tgt = tgt.trim();

						/* find the target location and adjust counters*/
						Enumeration locs_2 = mac.llist.elements(); /*location list for this machine */
						while(locs_2.hasMoreElements())
						{

							_location loc_2 = (_location)locs_2.nextElement();
							if(tgt.equals(loc_2.name))
							{
								//System.out.println("No RESET.... from loc "+loc.name+" to loc: "+tran.target);
								loc_2.entering_trans++;
							}
						}
					} //end else
				}//end inner while
			}//end outer while
		}// end machine loop
	}//end function

	static void print_mod(_model mod){

		System.out.println("Printing model ********");
		Enumeration macs = mod.mac_list.elements();

		while(macs.hasMoreElements())
		{
			_machine mac=(_machine) macs.nextElement();
			System.out.println("machine");
			System.out.println(mac.name);
			System.out.println(mac.llist.size());
		    System.out.println(mac.ini_loc);

			Enumeration locs = mac.llist.elements(); /*location list for this machine */
			while(locs.hasMoreElements()){

				_location loc=(_location)locs.nextElement();

				System.out.println("location: " + loc.name + " " +loc.reset_flag);
				Enumeration trans=loc.tlist.elements();
				while(trans.hasMoreElements()){
					_transition tran=(_transition)trans.nextElement();

					System.out.println(" target: " + tran.target + " guard: " + tran.guard);
					if(tran.reset.vecs.size()>0)
					{
						System.out.println("Reset present");
					}
					else{
						System.out.println("no reset present");
					}
				}
				System.out.println(" ");
			}
			System.out.println("***********");
		}
	}
}

